@@ -1,8 +1,9 @@
+
+
 package com.unciv.logic.map.mapunit.movement
 
 import com.badlogic.gdx.math.Vector2
 import com.unciv.Constants
-import com.unciv.UncivGame
 import com.unciv.logic.map.BFS
 import com.unciv.logic.map.HexMath.getDistance
 import com.unciv.logic.map.mapunit.MapUnit
@@ -10,14 +11,8 @@ import com.unciv.logic.map.tile.Tile
 import com.unciv.models.UnitActionType
 import com.unciv.models.ruleset.unique.UniqueType
 import com.unciv.ui.components.UnitMovementMemoryType
-import java.util.TreeSet
 
 
-fun List<UnitMovement.MovementStep>.toBackwardsCompatiblePath(): List<Tile> {
-    val backwardsCompatiblePath = this.reversed().filter { it.totalCost.movementLeft == 0f || it == this.last() }
-    return backwardsCompatiblePath.map { it.tile }
-}
-
 class UnitMovement(val unit: MapUnit) {
 
     private val pathfindingCache = PathfindingCache(unit)
@@ -31,13 +26,13 @@ class UnitMovement(val unit: MapUnit) {
      * If a tile can be reached within the turn, but it cannot be passed through, the total distance to it is set to unitMovement
      */
     fun getDistanceToTilesWithinTurn(
-                origin: Vector2,
-                unitMovement: Float,
-                considerZoneOfControl: Boolean = true,
-                tilesToIgnore: HashSet<Tile>? = null,
-                passThroughCache: HashMap<Tile, Boolean> = HashMap(),
-                movementCostCache: HashMap<Pair<Tile, Tile>, Float> = HashMap()
-            ): PathsToTilesWithinTurn {
+        origin: Vector2,
+        unitMovement: Float,
+        considerZoneOfControl: Boolean = true,
+        tilesToIgnore: HashSet<Tile>? = null,
+        passThroughCache: HashMap<Tile, Boolean> = HashMap(),
+        movementCostCache: HashMap<Pair<Tile, Tile>, Float> = HashMap()
+    ): PathsToTilesWithinTurn {
         val distanceToTiles = PathsToTilesWithinTurn()
 
         val currentUnitTile = unit.currentTile
@@ -65,9 +60,9 @@ class UnitMovement(val unit: MapUnit) {
                         else -> {
                             val key = Pair(tileToCheck, neighbor)
                             val movementCost =
-                                    movementCostCache.getOrPut(key) {
-                                        MovementCost.getMovementCostBetweenAdjacentTiles(unit, tileToCheck, neighbor, considerZoneOfControl)
-                                    }
+                                movementCostCache.getOrPut(key) {
+                                    MovementCost.getMovementCostBetweenAdjacentTiles(unit, tileToCheck, neighbor, considerZoneOfControl)
+                                }
                             distanceToTiles[tileToCheck]!!.totalDistance + movementCost
                         }
                     }
@@ -90,139 +85,11 @@ class UnitMovement(val unit: MapUnit) {
         return distanceToTiles
     }
 
-
-    data class MovementStep(val previousStep: MovementStep?, val tile: Tile, val movementCost:Float, val totalCost: MovementStepTotalCost)
-
-
-    data class MovementStepTotalCost(/** Turn 0 means the initial turn */ val turn: Int, val movementLeft: Float):Comparable<MovementStepTotalCost> {
-        override operator fun compareTo(other: MovementStepTotalCost) =
-            compareValuesBy(this, other, {it.turn}, {-it.movementLeft})
-    }
-
-    /** Problem and solution documented at https://yairm210.medium.com/multi-turn-pathfinding-7136bd0bdaf0 */
-    fun getShortestPathNew(destination: Tile, considerZoneOfControl: Boolean = true,
-                           /** For allowing optional avoid of damaging tiles, tiles outside borders, etc */ shouldAvoidTile: (Tile) -> Boolean = {false},
-                           maxTurns: Int = 25,
-    ): List<MovementStep> {
-        if (unit.cache.cannotMove) return listOf()
-
-        val startingTile = unit.getTile()
-        val initialStep = MovementStep(null, startingTile, 0f, MovementStepTotalCost(0, unit.currentMovement))
-
-        if (startingTile.position == destination) {
-            // edge case that's needed, so that workers will know that they can reach their own tile. *sigh*
-            pathfindingCache.setShortestPathCache(destination, listOf(startingTile))
-            return listOf(initialStep)
-        }
-
-        val tileToBestStep = HashMap<Tile, MovementStep>() // contains a map of "you can get from X to Y in that turn"
-        tileToBestStep[startingTile] = initialStep
-
-        val tilesToCheck = TreeSet { t: Tile, t2: Tile ->
-            val tStep = tileToBestStep[t]!!
-            val t2Step = tileToBestStep[t2]!!
-            // This last comparitor is REQUIRED otherwise the tree will think that tiles the same distance away are the same and will throw the second one away!
-            compareValuesBy(tStep, t2Step, {it.totalCost}, {it.tile.aerialDistanceTo(destination)}, {it.tile.position.hashCode()})
-        }
-
-        tilesToCheck.add(startingTile)
-        val canEndTurnInCache = HashMap<Tile, Boolean>()
-        val unitMaxMovement = unit.getMaxMovement().toFloat()
-        val movementCostCache = HashMap<Pair<Tile, Tile>, Float>()
-        val shouldAvoidTileCache = HashMap<Tile, Boolean>()
-        val canPassThroughCache = HashMap<Tile, Boolean>()
-
-        while (tilesToCheck.isNotEmpty()){
-            val currentTileToCheck = tilesToCheck.pollFirst()!!
-
-            val currentTileStep = tileToBestStep[currentTileToCheck]!!
-
-            for (neighbor in currentTileToCheck.neighbors){
-                if (shouldAvoidTileCache.getOrPut(neighbor){ shouldAvoidTile(neighbor) }) continue
-                val currentBestStepToNeighbor = tileToBestStep[neighbor]
-                // If this tile can't beat the current best then no point checking
-                if (currentBestStepToNeighbor!=null && (currentBestStepToNeighbor.totalCost < currentTileStep.totalCost))
-                    continue
-
-                if (!canPassThroughCache.getOrPut(neighbor){ canPassThrough(neighbor) }) continue
-
-                val movementBetweenTiles: Float = if (!neighbor.isExplored(unit.civ)) 1f  // If we don't know then we just guess it to be 1.
-                else movementCostCache.getOrPut(currentTileToCheck to neighbor) {
-                    MovementCost.getMovementCostBetweenAdjacentTiles(unit, currentTileToCheck, neighbor, considerZoneOfControl)
-                }
-
-                val newStep = getNextStep(currentTileStep, neighbor, movementBetweenTiles, canEndTurnInCache, unitMaxMovement)
-                    ?: continue
-
-
-                if (neighbor == destination){
-                    val entirePath = arrayListOf<MovementStep>()
-                    var currentStep = newStep
-                    // We do NOT include the origin tile in this list
-                    while (currentStep.previousStep != null){
-                        entirePath.add(currentStep)
-                        currentStep = currentStep.previousStep!!
-                    }
-                    return entirePath.reversed()
-                }
-
-                if (currentBestStepToNeighbor == null ||
-                    newStep.totalCost < currentBestStepToNeighbor.totalCost) { // We have a winner!
-                    tileToBestStep[neighbor] = newStep
-                    if (newStep.totalCost.movementLeft == 0f && newStep.totalCost.turn == maxTurns) continue // don't schedule further expansion
-                    tilesToCheck.add(neighbor)
-                }
-            }
-        }
-
-        return listOf() // no path
-    }
-
-    fun getNextStep(currentTileStep:MovementStep, neighbor:Tile, movementBetweenTiles:Float,
-                    canEndTurnInCache:HashMap<Tile, Boolean>, unitMaxMovement:Float):MovementStep? {
-
-        fun canEndTurnIn(tile:Tile) = canEndTurnInCache.getOrPut(tile) { unit.movement.canMoveTo(tile) }
-
-        fun Float.normalizeMovementLeft() = if (this > Constants.minimumMovementEpsilon) this else 0f
-
-        val currentTile = currentTileStep.tile
-
-        // We can move directly
-        if (currentTileStep.totalCost.movementLeft != 0f || canEndTurnIn(currentTile)) {
-            val newTotalCost = if (currentTileStep.totalCost.movementLeft == 0f) MovementStepTotalCost(
-                currentTileStep.totalCost.turn + 1,
-                (unitMaxMovement - movementBetweenTiles).normalizeMovementLeft()
-            )
-            else MovementStepTotalCost(currentTileStep.totalCost.turn, (currentTileStep.totalCost.movementLeft - movementBetweenTiles).normalizeMovementLeft())
-            return MovementStep(currentTileStep, neighbor, movementBetweenTiles, newTotalCost)
-        }
-
-        // Backtracking nonsense - we CANNOT end the turn on the previous tile, AND we have no movement left, that means we need to do some alternate history
-        val previousStep = currentTileStep.previousStep ?: return null
-        if (previousStep.totalCost.movementLeft == 0f) return null // We backtracked until a previous end-of-turn and couldn't find any intermediate tiles
-        if (!canEndTurnIn(previousStep.tile)) return null
-        // We found somewhere we could have rested - let's do some alternate history!
-        val currentTileAlternateHistory = MovementStep(currentTileStep.previousStep, currentTile, currentTileStep.movementCost,
-            MovementStepTotalCost(currentTileStep.previousStep.totalCost.turn+1, (unitMaxMovement - currentTileStep.movementCost).normalizeMovementLeft()))
-        if (currentTileAlternateHistory.totalCost.movementLeft == 0f) return null // We tried, and even if we rest there we STILL have to stop at current tile... :/
-        val newTotalCost = MovementStepTotalCost(currentTileAlternateHistory.totalCost.turn, (currentTileAlternateHistory.totalCost.movementLeft-currentTileStep.movementCost).normalizeMovementLeft())
-        return MovementStep(currentTileAlternateHistory, neighbor, movementBetweenTiles, newTotalCost)
-    }
-
     /**
      * Does not consider if the [destination] tile can actually be entered, use [canMoveTo] for that.
      * Returns an empty list if there's no way to get to the destination.
      */
     fun getShortestPath(destination: Tile, avoidDamagingTerrain: Boolean = false): List<Tile> {
-        if (UncivGame.Current.settings.experimentalMovement) {
-            fun shouldAvoidEnemyTile(tile:Tile) = unit.isCivilian() && unit.isAutomated() && tile.isEnemyTerritory(unit.civ)
-            if (avoidDamagingTerrain){
-                val shortestPathWithoutDamagingTiles = getShortestPathNew(destination,
-                    shouldAvoidTile = { shouldAvoidEnemyTile(it) || unit.getDamageFromTerrain(it) > 0 })
-                if (shortestPathWithoutDamagingTiles.isNotEmpty()) return shortestPathWithoutDamagingTiles.toBackwardsCompatiblePath()
-            }
-            return getShortestPathNew(destination, shouldAvoidTile = ::shouldAvoidEnemyTile).toBackwardsCompatiblePath()
-        }
         if (unit.cache.cannotMove) return listOf()
 
         // First try and find a path without damaging terrain
@@ -231,7 +98,7 @@ class UnitMovement(val unit: MapUnit) {
             if (damageFreePath.isNotEmpty()) return damageFreePath
         }
         if (unit.baseUnit.isWaterUnit()
-                && destination.neighbors.none { isUnknownTileWeShouldAssumeToBePassable(it) || it.isWater }) {
+            && destination.neighbors.none { isUnknownTileWeShouldAssumeToBePassable(it) || it.isWater }) {
             // edge case where this unit is a boat and all of the tiles around the destination are
             // explored and known to be land so we know a priori that no path exists
             pathfindingCache.setShortestPathCache(destination, listOf())
@@ -306,8 +173,8 @@ class UnitMovement(val unit: MapUnit) {
                     } else {
                         if (movementTreeParents.containsKey(reachableTile)) continue // We cannot be faster than anything existing...
                         if (!isUnknownTileWeShouldAssumeToBePassable(reachableTile) &&
-                                !canMoveToCache.getOrPut(reachableTile) { canMoveTo(reachableTile) })
-                            // This is a tile that we can't actually enter - either an intermediary tile containing our unit, or an enemy unit/city
+                            !canMoveToCache.getOrPut(reachableTile) { canMoveTo(reachableTile) })
+                        // This is a tile that we can't actually enter - either an intermediary tile containing our unit, or an enemy unit/city
                             continue
                         movementTreeParents[reachableTile] = tileToCheck
                         newTilesToCheck.add(reachableTile)
@@ -345,9 +212,13 @@ class UnitMovement(val unit: MapUnit) {
         val distanceToTiles = getDistanceToTiles()
 
         // If the tile is far away, we need to build a path how to get there, and then take the first step
-        if (!distanceToTiles.containsKey(finalDestination))
-            return getShortestPath(finalDestination).firstOrNull()
+        if (!distanceToTiles.containsKey(finalDestination)) {
+            val shortestDestination = getShortestPath(finalDestination).firstOrNull()
                 ?: throw UnreachableDestinationException("$unit ${unit.currentTile} cannot reach $finalDestination")
+            if (shortestDestination !in distanceToTiles)
+                return distanceToTiles.keys.minBy { it.aerialDistanceTo(finalDestination) }
+            return shortestDestination
+        }
 
         // we should be able to get there this turn
         if (canMoveTo(finalDestination))
@@ -360,7 +231,7 @@ class UnitMovement(val unit: MapUnit) {
             else -> destinationNeighbors
                 .filter { distanceToTiles.containsKey(it) && canMoveTo(it) }
                 .minByOrNull { distanceToTiles.getValue(it).totalDistance } // we can get a little closer
-                    ?: currentTile // We can't get closer...
+                ?: currentTile // We can't get closer...
         }
     }
 
@@ -431,11 +302,11 @@ class UnitMovement(val unit: MapUnit) {
                 reachableTile.civilianUnit
             else
                 reachableTile.militaryUnit
-        ) ?: return false
+            ) ?: return false
         val ourPosition = unit.getTile()
         if (otherUnit.owner != unit.owner
-                || otherUnit.cache.cannotMove  // redundant, line below would cover it too
-                || !otherUnit.movement.canReachInCurrentTurn(ourPosition)) return false
+            || otherUnit.cache.cannotMove  // redundant, line below would cover it too
+            || !otherUnit.movement.canReachInCurrentTurn(ourPosition)) return false
 
         if (!canMoveTo(reachableTile, canSwap = true)) return false
         if (!otherUnit.movement.canMoveTo(ourPosition, canSwap = true)) return false
@@ -633,7 +504,7 @@ class UnitMovement(val unit: MapUnit) {
                 destination.civilianUnit
             else
                 destination.militaryUnit
-        )?: return // The precondition guarantees that there is an eligible same-type unit at the destination
+            )?: return // The precondition guarantees that there is an eligible same-type unit at the destination
 
         val ourOldPosition = unit.getTile()
         val theirOldPosition = otherUnit.getTile()
@@ -696,7 +567,7 @@ class UnitMovement(val unit: MapUnit) {
         else
         // can skip checking for airUnit since not a city
             (tile.militaryUnit == null || (canSwap && tile.militaryUnit!!.owner == unit.owner))
-            && (tile.civilianUnit == null || tile.civilianUnit!!.owner == unit.owner || unit.civ.isAtWarWith(tile.civilianUnit!!.civ))
+                && (tile.civilianUnit == null || tile.civilianUnit!!.owner == unit.owner || unit.civ.isAtWarWith(tile.civilianUnit!!.civ))
     }
 
     private fun canAirUnitMoveTo(tile: Tile, unit: MapUnit): Boolean {
@@ -741,14 +612,14 @@ class UnitMovement(val unit: MapUnit) {
                 return false
         }
         if (tile.isLand
-                && unit.baseUnit.isWaterUnit()
-                && !tile.isCityCenter())
+            && unit.baseUnit.isWaterUnit()
+            && !tile.isCityCenter())
             return false
 
         val unitSpecificAllowOcean: Boolean by lazy {
             unit.civ.tech.specificUnitsCanEnterOcean &&
-                    unit.civ.getMatchingUniques(UniqueType.UnitsMayEnterOcean)
-                        .any { unit.matchesFilter(it.params[0]) }
+                unit.civ.getMatchingUniques(UniqueType.UnitsMayEnterOcean)
+                    .any { unit.matchesFilter(it.params[0]) }
         }
         if (tile.isWater && unit.baseUnit.isLandUnit() && !unit.cache.canMoveOnWater) {
             if (!unit.civ.tech.unitsCanEmbark) return false
@@ -759,7 +630,7 @@ class UnitMovement(val unit: MapUnit) {
             if (!unitSpecificAllowOcean && unit.cache.cannotEnterOceanTiles) return false
         }
 
-        if (unit.hasUnique(UniqueType.CanTradeWithCityStateForGoldAndInfluence) && tile.getOwner()?.isCityState() == true)
+        if (unit.cache.canEnterCityStates && tile.getOwner()?.isCityState() == true)
             return true
         if (!unit.cache.canEnterForeignTerrain && !tile.canCivPassThrough(unit.civ)) return false
 
@@ -773,7 +644,7 @@ class UnitMovement(val unit: MapUnit) {
             // Allow movement through unguarded, at-war Civilian Unit. Capture on the way
             // But not for Embarked Units capturing on Water
             if (!(unit.baseUnit.isLandUnit() && tile.isWater && !unit.cache.canMoveOnWater)
-                    && firstUnit.isCivilian() && unit.civ.isAtWarWith(firstUnit.civ))
+                && firstUnit.isCivilian() && unit.civ.isAtWarWith(firstUnit.civ))
                 return true
             // Cannot enter hostile tile with any unit in there
             if (unit.civ.isAtWarWith(firstUnit.civ))
@@ -785,10 +656,10 @@ class UnitMovement(val unit: MapUnit) {
 
 
     fun getDistanceToTiles(
-                considerZoneOfControl: Boolean = true,
-                passThroughCache: HashMap<Tile, Boolean> = HashMap(),
-                movementCostCache: HashMap<Pair<Tile, Tile>, Float> = HashMap())
-            : PathsToTilesWithinTurn {
+        considerZoneOfControl: Boolean = true,
+        passThroughCache: HashMap<Tile, Boolean> = HashMap(),
+        movementCostCache: HashMap<Pair<Tile, Tile>, Float> = HashMap())
+        : PathsToTilesWithinTurn {
         val cacheResults = pathfindingCache.getDistanceToTiles(considerZoneOfControl)
         if (cacheResults != null) {
             return cacheResults
@@ -819,7 +690,7 @@ class UnitMovement(val unit: MapUnit) {
             val newTilesToCheck = ArrayList<Tile>()
             for (currentTileToCheck in tilesToCheck) {
                 val reachableTiles = currentTileToCheck.getTilesInDistance(unit.getRange())
-                        .filter { unit.movement.canMoveTo(it) }
+                    .filter { unit.movement.canMoveTo(it) }
                 for (reachableTile in reachableTiles) {
                     if (tilesReached.containsKey(reachableTile)) continue
                     tilesReached[reachableTile] = currentTileToCheck
