@@ -6,7 +6,6 @@ import com.unciv.logic.civilization.PlayerType
 
 class Player(
     var chosenCiv: String = Constants.random,
-    var playerType: PlayerType = PlayerType.AI
-) : IsPartOfGameInfoSerialization {
-    var playerId = ""
-}
+    var playerType: PlayerType = PlayerType.AI,
+    var playerId:String = ""
+) : IsPartOfGameInfoSerialization
